searchData={"items":[{"type":"module","title":"KafkaBatcher","doc":"```mermaid\nflowchart TD\n    S -->|one for all collectors| CM(ConnectionManager)\n    S -->|reg_name by topic| APS[AccumulatorsPoolSupervisor]\n    S[Supervisor] -->|reg_name by topic| C(Collector)\n    APS --> Accumulator0\n    APS -->|starts Accumulator for each partition| Accumulator1\n    APS --> Accumulator2\n```\n\nor only one Accumulator in case when `collect_by_partition: false`\n```mermaid\nflowchart TD\n    S -->|one for all collectors| CM(ConnectionManager)\n    S -->|reg_name by topic| APS[AccumulatorsPoolSupervisor]\n    S[Supervisor] -->|reg_name by topic| C(Collector)\n    APS --> Accumulator\n```\n\nSequence in case when Kafka is avaliable:\n```mermaid\nsequenceDiagram\n    actor U as LibraryUser\n    participant C as Collector\n    participant A as Accumulator\n    participant P as Producer\n    U ->> C: add_events/1\n    C->>A: dispatch by partitions & add_event/3\n    A->>A: accumulate events\n    A->>P: when conditions met call producer\n    P->>Kafka: produce_sync\n    Kafka->>P: :ok\n    P->>A: :ok\n    A->>A: reset_state_after_produce/2\n```\n\nSequence in case when Kafka is unavaliable:\n```mermaid\nsequenceDiagram\n    actor U as LibraryUser\n    participant C as Collector\n    participant A as Accumulator\n    participant P as Producer\n    U ->> C: add_events/1\n    C->>A: dispatch by partitions & add_event/3\n    A->>A: accumulate events\n    A->>P: when conditions met call producer\n    P->>Kafka: produce_sync\n    Kafka->>P: unavailable\n    P->>A: {:error, reason}\n    A->>TempStorage: save_batch/1\n```","ref":"KafkaBatcher.html"},{"type":"module","title":"KafkaBatcher.Accumulator","doc":"Accumulator process is used to accumulate messages until Accumulator.State will go into \"ready to producing\" status.\nThere are many conditions to detect this status, which can be configured through kafka_batcher settings.\nSee details how it works in KafkaBatcher.Accumulator.State module","ref":"KafkaBatcher.Accumulator.html"},{"type":"function","title":"KafkaBatcher.Accumulator.add_event/3","doc":"Finds appropriate Accumulator process by topic & partition and dispatches `event` to it","ref":"KafkaBatcher.Accumulator.html#add_event/3"},{"type":"function","title":"KafkaBatcher.Accumulator.child_spec/1","doc":"Returns a specification to start this module under a supervisor","ref":"KafkaBatcher.Accumulator.html#child_spec/1"},{"type":"function","title":"KafkaBatcher.Accumulator.start_link/1","doc":"","ref":"KafkaBatcher.Accumulator.html#start_link/1"},{"type":"module","title":"KafkaBatcher.Accumulator.DefaultBatchFlusher","doc":"This module implements logic for force pushing current batch to Kafka,\nwithout waiting for other conditions (on size and/or interval).","ref":"KafkaBatcher.Accumulator.DefaultBatchFlusher.html"},{"type":"module","title":"KafkaBatcher.Accumulator.State","doc":"Encapsulates all logic to detect when batch will be ready to producing\n\nA batch is marked as ready for producing when one of the following conditions is met:\n\n  * reached the max byte size of the batch\n  * reached the batch size (messages count) limit\n  * reached the waiting time limit (max delay before producing)\n  * one of special events arrived (which triggers Flusher to produce immediately)\n  * timer expired (in case when a few events arrived timer helps to control that the max waiting time is not exceeded)","ref":"KafkaBatcher.Accumulator.State.html"},{"type":"function","title":"KafkaBatcher.Accumulator.State.add_new_message/3","doc":"","ref":"KafkaBatcher.Accumulator.State.html#add_new_message/3"},{"type":"function","title":"KafkaBatcher.Accumulator.State.mark_as_ready/1","doc":"","ref":"KafkaBatcher.Accumulator.State.html#mark_as_ready/1"},{"type":"function","title":"KafkaBatcher.Accumulator.State.reset_state_after_failure/1","doc":"","ref":"KafkaBatcher.Accumulator.State.html#reset_state_after_failure/1"},{"type":"function","title":"KafkaBatcher.Accumulator.State.reset_state_after_produce/1","doc":"","ref":"KafkaBatcher.Accumulator.State.html#reset_state_after_produce/1"},{"type":"type","title":"KafkaBatcher.Accumulator.State.t/0","doc":"","ref":"KafkaBatcher.Accumulator.State.html#t:t/0"},{"type":"module","title":"KafkaBatcher.AccumulatorsPoolSupervisor","doc":"Manage dynamic pool of accumulators for the KafkaBatcher.Collector instance","ref":"KafkaBatcher.AccumulatorsPoolSupervisor.html"},{"type":"function","title":"KafkaBatcher.AccumulatorsPoolSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor","ref":"KafkaBatcher.AccumulatorsPoolSupervisor.html#child_spec/1"},{"type":"function","title":"KafkaBatcher.AccumulatorsPoolSupervisor.init/1","doc":"","ref":"KafkaBatcher.AccumulatorsPoolSupervisor.html#init/1"},{"type":"function","title":"KafkaBatcher.AccumulatorsPoolSupervisor.reg_name/1","doc":"","ref":"KafkaBatcher.AccumulatorsPoolSupervisor.html#reg_name/1"},{"type":"function","title":"KafkaBatcher.AccumulatorsPoolSupervisor.start_accumulator/1","doc":"","ref":"KafkaBatcher.AccumulatorsPoolSupervisor.html#start_accumulator/1"},{"type":"function","title":"KafkaBatcher.AccumulatorsPoolSupervisor.start_link/1","doc":"","ref":"KafkaBatcher.AccumulatorsPoolSupervisor.html#start_link/1"},{"type":"behaviour","title":"KafkaBatcher.Behaviours.BatchFlusher","doc":"Determines whether we have to produce the batch immediately","ref":"KafkaBatcher.Behaviours.BatchFlusher.html"},{"type":"callback","title":"KafkaBatcher.Behaviours.BatchFlusher.flush?/2","doc":"","ref":"KafkaBatcher.Behaviours.BatchFlusher.html#c:flush?/2"},{"type":"behaviour","title":"KafkaBatcher.Behaviours.Collector","doc":"Collector must implement add_events/1 callback to receive events.\n\nEvent could be a MessageObject or a tuple with headers, key and value, but headers and key could be omitted","ref":"KafkaBatcher.Behaviours.Collector.html"},{"type":"callback","title":"KafkaBatcher.Behaviours.Collector.add_events/1","doc":"","ref":"KafkaBatcher.Behaviours.Collector.html#c:add_events/1"},{"type":"type","title":"KafkaBatcher.Behaviours.Collector.event/0","doc":"","ref":"KafkaBatcher.Behaviours.Collector.html#t:event/0"},{"type":"type","title":"KafkaBatcher.Behaviours.Collector.events/0","doc":"","ref":"KafkaBatcher.Behaviours.Collector.html#t:events/0"},{"type":"type","title":"KafkaBatcher.Behaviours.Collector.header_key/0","doc":"","ref":"KafkaBatcher.Behaviours.Collector.html#t:header_key/0"},{"type":"type","title":"KafkaBatcher.Behaviours.Collector.header_value/0","doc":"","ref":"KafkaBatcher.Behaviours.Collector.html#t:header_value/0"},{"type":"type","title":"KafkaBatcher.Behaviours.Collector.headers/0","doc":"","ref":"KafkaBatcher.Behaviours.Collector.html#t:headers/0"},{"type":"type","title":"KafkaBatcher.Behaviours.Collector.key/0","doc":"","ref":"KafkaBatcher.Behaviours.Collector.html#t:key/0"},{"type":"type","title":"KafkaBatcher.Behaviours.Collector.message_object/0","doc":"","ref":"KafkaBatcher.Behaviours.Collector.html#t:message_object/0"},{"type":"type","title":"KafkaBatcher.Behaviours.Collector.value/0","doc":"","ref":"KafkaBatcher.Behaviours.Collector.html#t:value/0"},{"type":"behaviour","title":"KafkaBatcher.Behaviours.ErrorNotifier","doc":"ErrorNotifier behaviour is used to report errors to external error-monitoring systems.\n\nIn the most base implementation it could just log errors.","ref":"KafkaBatcher.Behaviours.ErrorNotifier.html"},{"type":"callback","title":"KafkaBatcher.Behaviours.ErrorNotifier.report/2","doc":"","ref":"KafkaBatcher.Behaviours.ErrorNotifier.html#c:report/2"},{"type":"behaviour","title":"KafkaBatcher.Behaviours.Producer","doc":"KafkaBatcher.Behaviours.Producer adds an abstraction level over producer implementations in various Kafka libs.\nDefines the callbacks that a Kafka producer should implement","ref":"KafkaBatcher.Behaviours.Producer.html"},{"type":"callback","title":"KafkaBatcher.Behaviours.Producer.do_produce/4","doc":"","ref":"KafkaBatcher.Behaviours.Producer.html#c:do_produce/4"},{"type":"callback","title":"KafkaBatcher.Behaviours.Producer.get_partitions_count/1","doc":"","ref":"KafkaBatcher.Behaviours.Producer.html#c:get_partitions_count/1"},{"type":"callback","title":"KafkaBatcher.Behaviours.Producer.start_client/0","doc":"","ref":"KafkaBatcher.Behaviours.Producer.html#c:start_client/0"},{"type":"callback","title":"KafkaBatcher.Behaviours.Producer.start_producer/2","doc":"","ref":"KafkaBatcher.Behaviours.Producer.html#c:start_producer/2"},{"type":"type","title":"KafkaBatcher.Behaviours.Producer.event/0","doc":"","ref":"KafkaBatcher.Behaviours.Producer.html#t:event/0"},{"type":"type","title":"KafkaBatcher.Behaviours.Producer.events/0","doc":"","ref":"KafkaBatcher.Behaviours.Producer.html#t:events/0"},{"type":"behaviour","title":"KafkaBatcher.Behaviours.TempStorage","doc":"KafkaBatcher.Behaviours.TempStorage behaviour is used to implement events saving logic in case when Kafka is not available.","ref":"KafkaBatcher.Behaviours.TempStorage.html"},{"type":"callback","title":"KafkaBatcher.Behaviours.TempStorage.empty?/1","doc":"Check if the storage is empty","ref":"KafkaBatcher.Behaviours.TempStorage.html#c:empty?/1"},{"type":"callback","title":"KafkaBatcher.Behaviours.TempStorage.save_batch/1","doc":"Save batch to retry","ref":"KafkaBatcher.Behaviours.TempStorage.html#c:save_batch/1"},{"type":"type","title":"KafkaBatcher.Behaviours.TempStorage.topic_name/0","doc":"","ref":"KafkaBatcher.Behaviours.TempStorage.html#t:topic_name/0"},{"type":"module","title":"KafkaBatcher.Collector","doc":"Implementation of collector for incoming events.\nThe collector accumulates events in accordance with a given strategy using accumulators supervised by AccumulatorsPoolSupervisor.\nThe strategy is specified by the following parameters:\n* `:partition_strategy`, allows values: :random, :md5 or function (e.g. `fn _topic, _partitions_count, key, _value -> key end`)\n* `:partition_fn`, a function that takes 4 arguments and returns a number of partition (see example below)\n* `:collect_by_partition`, if set to `true`, producer accumulates messages separately for each partition of topic\n* `:batch_size`, count of messages to be accumulated before producing\n* `:max_wait_time`, max interval between producings in milliseconds. The batch will be produced to Kafka either by `batch_size` or by `max_wait_time` parameter.\n* `:batch_flusher`, a module implementing a `flush?/2` function. If the function returns true, the current batch will be sent to Kafka immediately.\n* `:min_delay` - optional parameter. Set minimal delay before send events. This parameter allowed to increase max throughput\n* `:max_batch_bytesize` - optional parameter. Allows to set a limit on the maximum batch size in bytes.\n\nA collector can be described as follows (for example):\n\n    defmodule KafkaBatcher.Test.Handler1 do\n      use KafkaBatcher.Collector,\n        collect_by_partition: true,\n        topic_key: :topic1,\n        partition_fn: &KafkaBatcher.Test.Handler1.calculate_partition/4,\n        required_acks: -1,\n        batch_size: 30,\n        max_wait_time: 20_000,\n        min_delay: 0\n\n      def calculate_partition(_topic, partitions_count, _key, value) do\n        val = value[\"client_id\"] || value[\"device_id\"]\n        :erlang.phash2(val, partitions_count)\n      end\n    end\n\nA collector can save events that cannot be sent to Kafka to external storage, such as a database.\nA storage is specified in the config.exs like this:\n\n    config :kafka_batcher,\n      storage_impl: KafkaBatcher.Storage.YourTempStorage","ref":"KafkaBatcher.Collector.html"},{"type":"module","title":"KafkaBatcher.Collector.Implementation","doc":"Part of the KafkaBatcher.Collector implementation not related to GenServer behavior.","ref":"KafkaBatcher.Collector.Implementation.html"},{"type":"function","title":"KafkaBatcher.Collector.Implementation.calculate_partition/3","doc":"","ref":"KafkaBatcher.Collector.Implementation.html#calculate_partition/3"},{"type":"function","title":"KafkaBatcher.Collector.Implementation.start_accumulators/1","doc":"","ref":"KafkaBatcher.Collector.Implementation.html#start_accumulators/1"},{"type":"module","title":"KafkaBatcher.Collector.State","doc":"Describes the state of KafkaBatcher.Collector and functions working with it","ref":"KafkaBatcher.Collector.State.html"},{"type":"function","title":"KafkaBatcher.Collector.State.add_events/2","doc":"","ref":"KafkaBatcher.Collector.State.html#add_events/2"},{"type":"module","title":"KafkaBatcher.Collector.Utils","doc":"Provides functions for transforming events in different input formats into MessageObject struct","ref":"KafkaBatcher.Collector.Utils.html"},{"type":"function","title":"KafkaBatcher.Collector.Utils.prepare_events/1","doc":"","ref":"KafkaBatcher.Collector.Utils.html#prepare_events/1"},{"type":"function","title":"KafkaBatcher.Collector.Utils.transform_event/1","doc":"","ref":"KafkaBatcher.Collector.Utils.html#transform_event/1"},{"type":"module","title":"KafkaBatcher.Config","doc":"KafkaBatcher configuration processing.\nAll config parameters are described in details in README.md\nExamples of configs can be found in the files config/test.exs and test/support/collectors/collector_handlers.ex","ref":"KafkaBatcher.Config.html"},{"type":"function","title":"KafkaBatcher.Config.build_topic_config/1","doc":"","ref":"KafkaBatcher.Config.html#build_topic_config/1"},{"type":"function","title":"KafkaBatcher.Config.collectors_spec/0","doc":"","ref":"KafkaBatcher.Config.html#collectors_spec/0"},{"type":"function","title":"KafkaBatcher.Config.general_producer_config/0","doc":"","ref":"KafkaBatcher.Config.html#general_producer_config/0"},{"type":"function","title":"KafkaBatcher.Config.get_collector_config/1","doc":"","ref":"KafkaBatcher.Config.html#get_collector_config/1"},{"type":"function","title":"KafkaBatcher.Config.get_configs_by_collector!/0","doc":"","ref":"KafkaBatcher.Config.html#get_configs_by_collector!/0"},{"type":"function","title":"KafkaBatcher.Config.get_configs_by_topic_name/0","doc":"Return all configured topics with its config.","ref":"KafkaBatcher.Config.html#get_configs_by_topic_name/0"},{"type":"function","title":"KafkaBatcher.Config.get_endpoints/0","doc":"","ref":"KafkaBatcher.Config.html#get_endpoints/0"},{"type":"function","title":"KafkaBatcher.Config.get_endpoints/1","doc":"","ref":"KafkaBatcher.Config.html#get_endpoints/1"},{"type":"type","title":"KafkaBatcher.Config.sasl_mechanism/0","doc":"","ref":"KafkaBatcher.Config.html#t:sasl_mechanism/0"},{"type":"type","title":"KafkaBatcher.Config.sasl_type/0","doc":"","ref":"KafkaBatcher.Config.html#t:sasl_type/0"},{"type":"module","title":"KafkaBatcher.ConnectionManager","doc":"Abstraction layer over Kafka library.\n\nManages a connection to Kafka, producers processes and does reconnect if something goes wrong.","ref":"KafkaBatcher.ConnectionManager.html"},{"type":"function","title":"KafkaBatcher.ConnectionManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor","ref":"KafkaBatcher.ConnectionManager.html#child_spec/1"},{"type":"function","title":"KafkaBatcher.ConnectionManager.client_started?/0","doc":"Checks that Kafka client is already started","ref":"KafkaBatcher.ConnectionManager.html#client_started?/0"},{"type":"function","title":"KafkaBatcher.ConnectionManager.start_link/0","doc":"","ref":"KafkaBatcher.ConnectionManager.html#start_link/0"},{"type":"module","title":"KafkaBatcher.ConnectionManager.State","doc":"State of ConnectionManager process","ref":"KafkaBatcher.ConnectionManager.State.html"},{"type":"type","title":"KafkaBatcher.ConnectionManager.State.t/0","doc":"","ref":"KafkaBatcher.ConnectionManager.State.html#t:t/0"},{"type":"module","title":"KafkaBatcher.DefaultErrorNotifier","doc":"Default implementation of the `KafkaBatcher.Behaviours.ErrorNotifier` behaviour.\n\nIt just logs the errors. You can implement your own implementation to send errors to some error monitoring system.","ref":"KafkaBatcher.DefaultErrorNotifier.html"},{"type":"module","title":"KafkaBatcher.MessageObject","doc":"Contains Kafka message fields","ref":"KafkaBatcher.MessageObject.html"},{"type":"module","title":"Fields - KafkaBatcher.MessageObject","doc":"* `:key` - Kafka use it for partitioning\n  * `:value` - the main payload of the message\n  * `:headers` - a keyword list with auxiliary key-value pairs","ref":"KafkaBatcher.MessageObject.html#module-fields"},{"type":"type","title":"KafkaBatcher.MessageObject.t/0","doc":"","ref":"KafkaBatcher.MessageObject.html#t:t/0"},{"type":"module","title":"KafkaBatcher.Producers.Base","doc":"General part of the Kafka producer implementation","ref":"KafkaBatcher.Producers.Base.html"},{"type":"module","title":"KafkaBatcher.Producers.Kaffe","doc":"An implementation of the KafkaBatcher.Behaviours.Producer for Kaffe","ref":"KafkaBatcher.Producers.Kaffe.html"},{"type":"function","title":"KafkaBatcher.Producers.Kaffe.produce_list/4","doc":"","ref":"KafkaBatcher.Producers.Kaffe.html#produce_list/4"},{"type":"module","title":"KafkaBatcher.Producers.KafkaEx","doc":"An implementation of the KafkaBatcher.Behaviours.Producer for KafkaEx","ref":"KafkaBatcher.Producers.KafkaEx.html"},{"type":"function","title":"KafkaBatcher.Producers.KafkaEx.produce_list/4","doc":"","ref":"KafkaBatcher.Producers.KafkaEx.html#produce_list/4"},{"type":"module","title":"KafkaBatcher.PromEx.Plugins.Kafka","doc":"PromEx plugin to collect Prometheus metrics of interactions with Kafka.\nThe following metrics are collected here:\n\n* prom_ex_kafka_producer_batch_total_size_byte_bucket\n* prom_ex_kafka_producer_batch_total_size_byte_sum\n* prom_ex_kafka_producer_batch_total_size_byte_count\n* prom_ex_kafka_producer_batch_messages_count_bucket\n* prom_ex_kafka_producer_batch_messages_count_sum\n* prom_ex_kafka_producer_batch_messages_count_count\n* prom_ex_kafka_producer_duration_seconds_bucket\n* prom_ex_kafka_producer_duration_seconds_sum\n* prom_ex_kafka_producer_duration_seconds_count\n\nEach metric has the following labels:\n* topic (topic name)\n* partition (partition number)\n* topic_alias (short name of topic to improve readability of Grafana dashboards in case when topic\n\nConfiguration options that allow you to set metrics display preferences:\n  :kafka_topic_aliases - allows you to set an alias for display in metrics\n  :producer_buckets - allows to set bucket parameters for grouping metrics\n  For example:\n\n     config :kafka_batcher,\n       kafka_topic_aliases: %{\n         my_topic1 => \"topic1\",\n         my_topic2 => \"topic2\"\n       }\n\n     config :kafka_batcher,\n       :kafka_metric_opts,\n         producer_buckets:\n         [\n           duration: [1, 2, 3, 4, 5, 10, 15, 20, 50, 100],\n           byte_size: [1_000, 2_000, 5_000, 10_000, 20_000, 50_000, 100_000],\n           messages_count: [1, 5, 10, 15, 20, 30, 40, 50, 100]\n         ]","ref":"KafkaBatcher.PromEx.Plugins.Kafka.html"},{"type":"function","title":"KafkaBatcher.PromEx.Plugins.Kafka.consumer_event_metrics/3","doc":"","ref":"KafkaBatcher.PromEx.Plugins.Kafka.html#consumer_event_metrics/3"},{"type":"function","title":"KafkaBatcher.PromEx.Plugins.Kafka.producer_event_metrics/3","doc":"","ref":"KafkaBatcher.PromEx.Plugins.Kafka.html#producer_event_metrics/3"},{"type":"module","title":"KafkaBatcher.Supervisor","doc":"The root of KafkaBatcher supervision tree\nStarts Collector & AccumulatorsPoolSupervisor for each configured collector","ref":"KafkaBatcher.Supervisor.html"},{"type":"function","title":"KafkaBatcher.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"KafkaBatcher.Supervisor.html#child_spec/1"},{"type":"function","title":"KafkaBatcher.Supervisor.init/1","doc":"","ref":"KafkaBatcher.Supervisor.html#init/1"},{"type":"function","title":"KafkaBatcher.Supervisor.start_link/1","doc":"","ref":"KafkaBatcher.Supervisor.html#start_link/1"},{"type":"module","title":"KafkaBatcher.TempStorage","doc":"Implements wrap-functions that are called to save batches when Kafka is unavailable.","ref":"KafkaBatcher.TempStorage.html"},{"type":"function","title":"KafkaBatcher.TempStorage.check_storage/1","doc":"","ref":"KafkaBatcher.TempStorage.html#check_storage/1"},{"type":"function","title":"KafkaBatcher.TempStorage.save_batch/1","doc":"","ref":"KafkaBatcher.TempStorage.html#save_batch/1"},{"type":"module","title":"KafkaBatcher.TempStorage.Batch","doc":"The struct used for KafkaBatcher.Behaviours.TempStorage behavior","ref":"KafkaBatcher.TempStorage.Batch.html"},{"type":"type","title":"KafkaBatcher.TempStorage.Batch.message/0","doc":"","ref":"KafkaBatcher.TempStorage.Batch.html#t:message/0"},{"type":"type","title":"KafkaBatcher.TempStorage.Batch.t/0","doc":"","ref":"KafkaBatcher.TempStorage.Batch.html#t:t/0"},{"type":"module","title":"KafkaBatcher.TempStorage.Default","doc":"Default implementation of KafkaBatcher.Behaviours.TempStorage\n\nIt just logs the messages. To have more fault tolerant implementation\nyou should implement your own logic to save these messages into some persistent storage.","ref":"KafkaBatcher.TempStorage.Default.html"},{"type":"exception","title":"KafkaBatcher.Config.BadConfigError","doc":"","ref":"KafkaBatcher.Config.BadConfigError.html"},{"type":"exception","title":"KafkaBatcher.Config.CollectorMissingError","doc":"","ref":"KafkaBatcher.Config.CollectorMissingError.html"},{"type":"exception","title":"KafkaBatcher.Config.SASLConfigError","doc":"","ref":"KafkaBatcher.Config.SASLConfigError.html"}],"content_type":"text/markdown"}